@@ -1,5 +1,8 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (C) Mellanox Technologies Ltd. 2001-2011.  ALL RIGHTS RESERVED.
+ * Copyright (c) 2015      Los Alamos National Security, LLC.  All rights
+ *                         reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -43,22 +46,21 @@ mca_mtl_mxm_component_t mca_mtl_mxm_component = {
      * First, the mca_base_component_t struct containing meta
      * information about the component itself
      */
-    {
+    .mtl_version = {
         MCA_MTL_BASE_VERSION_2_0_0,
         .mca_component_name = "mxm",
-        .mca_component_major_version = OMPI_MAJOR_VERSION,
-        .mca_component_minor_version = OMPI_MINOR_VERSION,
-        .mca_component_release_version = OMPI_RELEASE_VERSION,
+        MCA_BASE_MAKE_VERSION(component, OMPI_MAJOR_VERSION, OMPI_MINOR_VERSION,
+                              OMPI_RELEASE_VERSION),
         .mca_open_component = ompi_mtl_mxm_component_open,
         .mca_close_component = ompi_mtl_mxm_component_close,
         .mca_query_component = ompi_mtl_mxm_component_query,
         .mca_register_component_params = ompi_mtl_mxm_component_register,
     },
-    {
+    .mtl_data = {
         /* The component is not checkpoint ready */
         MCA_BASE_METADATA_PARAM_NONE
     },
-    ompi_mtl_mxm_component_init /* component init */
+    .mtl_init = ompi_mtl_mxm_component_init,
 }
 };
 
@@ -66,6 +68,12 @@ static int ompi_mtl_mxm_component_register(void)
 {
     mca_base_component_t*c;
 
+#if MXM_API < MXM_VERSION(3,0)
+    unsigned long cur_ver;
+    long major, minor;
+    char* runtime_version;
+#endif
+
     c = &mca_mtl_mxm_component.super.mtl_version;
 
     ompi_mtl_mxm.verbose = 0;
@@ -89,7 +97,41 @@ static int ompi_mtl_mxm_component_register(void)
                                            MCA_BASE_VAR_SCOPE_READONLY,
                                            &ompi_mtl_mxm.mxm_np);
 
-    param_priority = 100;
+    ompi_mtl_mxm.compiletime_version = MXM_VERNO_STRING;
+    (void) mca_base_component_var_register(c,
+            MCA_COMPILETIME_VER,
+            "Version of the libmxm library with which Open MPI was compiled",
+            MCA_BASE_VAR_TYPE_VERSION_STRING,
+            NULL, 0, 0,
+            OPAL_INFO_LVL_3,
+            MCA_BASE_VAR_SCOPE_READONLY,
+            &ompi_mtl_mxm.compiletime_version);
+
+#if MXM_API >= MXM_VERSION(3,0)
+    ompi_mtl_mxm.runtime_version = (char *)mxm_get_version_string();
+#else
+    cur_ver = mxm_get_version();
+    major = (cur_ver >> MXM_MAJOR_BIT) & 0xff;
+    minor = (cur_ver >> MXM_MINOR_BIT) & 0xff;
+    asprintf(&runtime_version, "%ld.%ld", major, minor);
+    ompi_mtl_mxm.runtime_version = runtime_version;
+#endif
+
+    (void) mca_base_component_var_register(c,
+            MCA_RUNTIME_VER,
+            "Version of the libmxm library with which Open MPI is running",
+            MCA_BASE_VAR_TYPE_VERSION_STRING,
+            NULL, 0, 0,
+            OPAL_INFO_LVL_3,
+            MCA_BASE_VAR_SCOPE_READONLY,
+            &ompi_mtl_mxm.runtime_version);
+
+#if MXM_API < MXM_VERSION(3,0)
+    free(runtime_version);
+#endif
+
+    /* set high enought to defeat ob1's default */
+    param_priority = 30;
     (void) mca_base_component_var_register (c,
                                             "priority", "Priority of the MXM MTL component",
                                             MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
@@ -150,9 +192,9 @@ static int ompi_mtl_mxm_component_open(void)
     cur_ver = mxm_get_version();
     if (cur_ver != MXM_API) {
         MXM_VERBOSE(1,
-                "WARNING: OMPI was compiled with MXM version %d.%d but version %ld.%ld detected.", 
+                "WARNING: OMPI was compiled with MXM version %d.%d but version %ld.%ld detected.",
                 MXM_VERNO_MAJOR,
-                MXM_VERNO_MINOR, 
+                MXM_VERNO_MINOR,
                 (cur_ver >> MXM_MAJOR_BIT) & 0xff,
                 (cur_ver >> MXM_MINOR_BIT) & 0xff);
     }
@@ -199,16 +241,16 @@ static int ompi_mtl_mxm_component_open(void)
         return OPAL_ERR_NOT_AVAILABLE;
     }
 
-    OBJ_CONSTRUCT(&mca_mtl_mxm_component.mxm_messages, ompi_free_list_t);
-    rc = ompi_free_list_init_new(&mca_mtl_mxm_component.mxm_messages,
-                                  sizeof(ompi_mtl_mxm_message_t),
-                                  opal_cache_line_size,
-                                  OBJ_CLASS(ompi_mtl_mxm_message_t),
-                                  0, opal_cache_line_size,
-                                  32 /* free list num */,
-                                  -1 /* free list max */,
-                                  32 /* free list inc */,
-                                  NULL);
+    OBJ_CONSTRUCT(&mca_mtl_mxm_component.mxm_messages, opal_free_list_t);
+    rc = opal_free_list_init (&mca_mtl_mxm_component.mxm_messages,
+                              sizeof(ompi_mtl_mxm_message_t),
+                              opal_cache_line_size,
+                              OBJ_CLASS(ompi_mtl_mxm_message_t),
+                              0, opal_cache_line_size,
+                              32 /* free list num */,
+                              -1 /* free list max */,
+                              32 /* free list inc */,
+                              NULL, 0, NULL, NULL, NULL);
     if (OMPI_SUCCESS != rc) {
         opal_show_help("help-mtl-mxm.txt", "mxm init", true,
                     mxm_error_string(err));
