@@ -1,16 +1,16 @@
 /* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2012      Sandia National Laboratories.  All rights reserved.
- * Copyright (c) 2014      Los Alamos National Security, LLC. All rights
+ * Copyright (c) 2014-2015 Los Alamos National Security, LLC. All rights
  *                         reserved.
  * Copyright (c) 2014      Intel, Inc. All rights reserved.
  * Copyright (c) 2015      Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2015-2016 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -19,7 +19,7 @@
 #include "ompi/mca/osc/osc.h"
 #include "ompi/mca/osc/base/base.h"
 #include "ompi/mca/osc/base/osc_base_obj_convert.h"
-#include "ompi/class/ompi_free_list.h"
+#include "ompi/request/request.h"
 #include "opal/util/sys_limits.h"
 
 #include "osc_sm.h"
@@ -37,23 +37,21 @@ static int component_select(struct ompi_win_t *win, void **base, size_t size, in
 
 ompi_osc_sm_component_t mca_osc_sm_component = {
     { /* ompi_osc_base_component_t */
-        { /* ompi_base_component_t */
+        .osc_version = {
             OMPI_OSC_BASE_VERSION_3_0_0,
-            "sm",
-            OMPI_MAJOR_VERSION,  /* MCA component major version */
-            OMPI_MINOR_VERSION,  /* MCA component minor version */
-            OMPI_RELEASE_VERSION,  /* MCA component release version */
-            component_open,
-            NULL
+            .mca_component_name = "sm",
+            MCA_BASE_MAKE_VERSION(component, OMPI_MAJOR_VERSION, OMPI_MINOR_VERSION,
+                                  OMPI_RELEASE_VERSION),
+            .mca_open_component = component_open,
         },
-        { /* mca_base_component_data */
+        .osc_data = { /* mca_base_component_data */
             /* The component is not checkpoint ready */
             MCA_BASE_METADATA_PARAM_NONE
         },
-        component_init,
-        component_query,
-        component_select,
-        component_finalize
+        .osc_init = component_init,
+        .osc_query = component_query,
+        .osc_select = component_select,
+        .osc_finalize = component_finalize,
     }
 };
 
@@ -117,7 +115,7 @@ component_init(bool enable_progress_threads, bool enable_mpi_threads)
 }
 
 
-static int 
+static int
 component_finalize(void)
 {
     /* clean up requests free list */
@@ -129,17 +127,13 @@ component_finalize(void)
 static int
 check_win_ok(ompi_communicator_t *comm, int flavor)
 {
-    int i;
-
     if (! (MPI_WIN_FLAVOR_SHARED == flavor
            || MPI_WIN_FLAVOR_ALLOCATE == flavor) ) {
         return OMPI_ERR_NOT_SUPPORTED;
     }
 
-    for (i = 0 ; i < ompi_comm_size(comm) ; ++i) {
-        if (!OPAL_PROC_ON_LOCAL_NODE(ompi_comm_peer_lookup(comm, i)->proc_flags)) {
-            return OMPI_ERR_RMA_SHARED;
-        }
+    if (ompi_group_have_remote_peers (comm->c_local_group)) {
+        return OMPI_ERR_RMA_SHARED;
     }
 
     return OMPI_SUCCESS;
@@ -184,7 +178,7 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
     OBJ_CONSTRUCT(&module->lock, opal_mutex_t);
 
     /* fill in the function pointer part */
-    memcpy(module, &ompi_osc_sm_module_template, 
+    memcpy(module, &ompi_osc_sm_module_template,
            sizeof(ompi_osc_base_module_t));
 
     /* need our communicator for collectives in next phase */
@@ -477,9 +471,11 @@ ompi_osc_sm_free(struct ompi_win_t *win)
         free(module->node_states);
         free(module->global_state);
         free(module->bases[0]);
-        free(module->bases);
-        free(module->sizes);
     }
+    free(module->disp_units);
+    free(module->outstanding_locks);
+    free(module->sizes);
+    free(module->bases);
 
     free (module->posts);
 
@@ -516,7 +512,7 @@ ompi_osc_sm_get_info(struct ompi_win_t *win, struct ompi_info_t **info_used)
     if (NULL == info) return OMPI_ERR_TEMP_OUT_OF_RESOURCE;
 
     if (module->flavor == MPI_WIN_FLAVOR_SHARED) {
-        ompi_info_set(info, "blocking_fence", 
+        ompi_info_set(info, "blocking_fence",
                       (1 == module->global_state->use_barrier_for_fence) ? "true" : "false");
         ompi_info_set(info, "alloc_shared_noncontig",
                       (module->noncontig) ? "true" : "false");
