@@ -1,5 +1,9 @@
 /*
- * Copyright (C) Mellanox Technologies Ltd. 2001-2011.  ALL RIGHTS RESERVED.
+ * Copyright (C) 2001-2011 Mellanox Technologies Ltd. ALL RIGHTS RESERVED.
+ * Copyright (c) 2015      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2015      Los Alamos National Security, LLC. All rights
+ *                         reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -7,13 +11,19 @@
  * $HEADER$
  */
 
+#ifdef HAVE_ALLOCA_H
+#include <alloca.h>
+#endif
+
 #include "pml_yalla.h"
 #include "pml_yalla_request.h"
 
-#include "ompi/runtime/ompi_module_exchange.h"
+#include "opal/runtime/opal.h"
+#include "opal/memoryhooks/memory.h"
+#include "opal/util/opal_environ.h"
+#include "opal/mca/pmix/pmix.h"
 #include "ompi/mca/pml/base/pml_base_bsend.h"
 #include "ompi/message/message.h"
-#include "opal/memoryhooks/memory.h"
 
 #define MODEX_KEY "yalla-mxm"
 
@@ -67,7 +77,8 @@ static int send_ep_address(void)
         return OMPI_ERROR;
     }
 
-    rc = ompi_modex_send(&mca_pml_yalla_component.pmlm_version, address, addrlen);
+    OPAL_MODEX_SEND(rc, OPAL_PMIX_GLOBAL,
+                    &mca_pml_yalla_component.pmlm_version, address, addrlen);
     if (OMPI_SUCCESS != rc) {
         PML_YALLA_ERROR("Open MPI couldn't distribute EP connection details");
         return OMPI_ERROR;
@@ -78,13 +89,14 @@ static int send_ep_address(void)
 
 static int recv_ep_address(ompi_proc_t *proc, void **address_p, size_t *addrlen_p)
 {
-    int ret = ompi_modex_recv(&mca_pml_yalla_component.pmlm_version, proc, address_p,
-                              addrlen_p);
+    int rc;
 
-    if (ret < 0) {
+    OPAL_MODEX_RECV(rc, &mca_pml_yalla_component.pmlm_version, &proc->super.proc_name,
+                    address_p, addrlen_p);
+    if (rc < 0) {
         PML_YALLA_ERROR("Failed to receive EP address");
     }
-    return ret;
+    return rc;
 }
 
 static void mca_pml_yalla_mem_release_cb(void *buf, size_t length,
@@ -175,6 +187,8 @@ int mca_pml_yalla_init(void)
     OBJ_CONSTRUCT(&ompi_pml_yalla.convs, mca_pml_yalla_freelist_t);
 
     opal_progress_register(mca_pml_yalla_progress);
+    
+    ompi_pml_yalla.super.pml_flags |= MCA_PML_BASE_FLAG_REQUIRE_WORLD;
 
     PML_YALLA_VERBOSE(2, "created mxm context %p ep %p", (void *)ompi_pml_yalla.mxm_context,
                       (void *)ompi_pml_yalla.mxm_ep);
@@ -225,11 +239,13 @@ int mca_pml_yalla_add_procs(struct ompi_proc_t **procs, size_t nprocs)
         }
 
         if (procs[i]->proc_endpoints[OMPI_PROC_ENDPOINT_TAG_PML]) {
-            PML_YALLA_VERBOSE(3, "already connected to proc. %d", procs[i]->proc_name.vpid);
+            PML_YALLA_VERBOSE(3, "already connected to proc. %s",
+                              OPAL_NAME_PRINT(procs[i]->super.proc_name));
             continue;
         }
 
-        PML_YALLA_VERBOSE(2, "connecting to proc. %d", procs[i]->proc_name.vpid);
+        PML_YALLA_VERBOSE(2, "connecting to proc. %s",
+                          OPAL_NAME_PRINT(procs[i]->super.proc_name));
         error = mxm_ep_connect(ompi_pml_yalla.mxm_ep, address, &conn);
         free(address);
 
@@ -247,22 +263,18 @@ int mca_pml_yalla_add_procs(struct ompi_proc_t **procs, size_t nprocs)
 int mca_pml_yalla_del_procs(struct ompi_proc_t **procs, size_t nprocs)
 {
     size_t i;
-    ompi_communicator_t *comm_world;
 
     if (ompi_mpi_finalized) {
         PML_YALLA_VERBOSE(3, "using bulk powerdown");
-        comm_world = &ompi_mpi_comm_world.comm;
-        if (ompi_comm_size(comm_world) > 1) {
-            comm_world->c_coll.coll_barrier(comm_world, comm_world->c_coll.coll_barrier_module);
-        }
         mxm_ep_powerdown(ompi_pml_yalla.mxm_ep);
     }
 
     for (i = 0; i < nprocs; ++i) {
         mxm_ep_disconnect(procs[i]->proc_endpoints[OMPI_PROC_ENDPOINT_TAG_PML]);
-        PML_YALLA_VERBOSE(2, "disconnected from rank %d", procs[i]->proc_name.vpid);
+        PML_YALLA_VERBOSE(2, "disconnected from rank %s", OPAL_NAME_PRINT(procs[i]->super.proc_name));
         procs[i]->proc_endpoints[OMPI_PROC_ENDPOINT_TAG_PML] = NULL;
     }
+    opal_pmix.fence(NULL, 0);
     return OMPI_SUCCESS;
 }
 
@@ -380,7 +392,7 @@ int mca_pml_yalla_recv(void *buf, size_t count, ompi_datatype_t *datatype, int s
     return OMPI_SUCCESS;
 }
 
-int mca_pml_yalla_isend_init(void *buf, size_t count, ompi_datatype_t *datatype,
+int mca_pml_yalla_isend_init(const void *buf, size_t count, ompi_datatype_t *datatype,
                              int dst, int tag, mca_pml_base_send_mode_t mode,
                              struct ompi_communicator_t* comm,
                              struct ompi_request_t **request)
@@ -434,11 +446,6 @@ static int mca_pml_yalla_bsend(mxm_send_req_t *mxm_sreq)
 
     bsreq->mxm.opcode         = mxm_sreq->opcode;
     bsreq->mxm.flags          = mxm_sreq->flags;
-#if MXM_API >= MXM_VERSION(3,4)
-    if (ompi_pml_yalla.force_bcopy_send) {
-        bsreq->mxm.flags     |= MXM_REQ_SEND_FLAG_BCOPY;
-    }
-#endif
     bsreq->mxm.op.send        = mxm_sreq->op.send;
 
     error = mxm_req_send(&bsreq->mxm);
@@ -452,7 +459,7 @@ static int mca_pml_yalla_bsend(mxm_send_req_t *mxm_sreq)
     return OMPI_SUCCESS;
 }
 
-int mca_pml_yalla_isend(void *buf, size_t count, ompi_datatype_t *datatype,
+int mca_pml_yalla_isend(const void *buf, size_t count, ompi_datatype_t *datatype,
                         int dst, int tag, mca_pml_base_send_mode_t mode,
                         struct ompi_communicator_t* comm,
                         struct ompi_request_t **request)
@@ -471,10 +478,8 @@ int mca_pml_yalla_isend(void *buf, size_t count, ompi_datatype_t *datatype,
 
     if (mode == MCA_PML_BASE_SEND_BUFFERED) {
         rc = mca_pml_yalla_bsend(&sreq->mxm);
-        OPAL_THREAD_LOCK(&ompi_request_lock);
         sreq->super.ompi.req_status.MPI_ERROR = rc;
         ompi_request_complete(&sreq->super.ompi, true);
-        OPAL_THREAD_UNLOCK(&ompi_request_lock);
         *request = &sreq->super.ompi;
         return rc;
     }
@@ -488,7 +493,7 @@ int mca_pml_yalla_isend(void *buf, size_t count, ompi_datatype_t *datatype,
     return OMPI_SUCCESS;
 }
 
-int mca_pml_yalla_send(void *buf, size_t count, ompi_datatype_t *datatype, int dst,
+int mca_pml_yalla_send(const void *buf, size_t count, ompi_datatype_t *datatype, int dst,
                        int tag, mca_pml_base_send_mode_t mode,
                        struct ompi_communicator_t* comm)
 {
@@ -701,10 +706,8 @@ int mca_pml_yalla_start(size_t count, ompi_request_t** requests)
             if (req->flags & MCA_PML_YALLA_REQUEST_FLAG_BSEND) {
                 PML_YALLA_VERBOSE(8, "start bsend request %p", (void *)sreq);
                 rc = mca_pml_yalla_bsend(&sreq->mxm);
-                OPAL_THREAD_LOCK(&ompi_request_lock);
                 sreq->super.ompi.req_status.MPI_ERROR = rc;
                 ompi_request_complete(&sreq->super.ompi, true);
-                OPAL_THREAD_UNLOCK(&ompi_request_lock);
                 if (OMPI_SUCCESS != rc) {
                     return rc;
                 }
