@@ -6,18 +6,20 @@
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2015      Los Alamos National Security, LLC.  All rights
  *                         reserved.
- * Copyright (c) 2015      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2015-2016 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2015      Intel, Inc. All rights reserved.
+ * Copyright (c) 2015      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 /*
@@ -54,20 +56,21 @@
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
-#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
-#endif
-#ifdef HAVE_STRING_H
 #include <string.h>
-#endif
 #ifdef HAVE_TARGETCONDITIONALS_H
 #include <TargetConditionals.h>
 #endif
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
+#ifdef HAVE_DLFCN_H
 #include <dlfcn.h>
+#endif
 #include <poll.h>
+#ifdef HAVE_LIBGEN_H
+#include <libgen.h>
+#endif
 
 #include "opal/util/output.h"
 #include "opal/datatype/opal_convertor.h"
@@ -80,9 +83,9 @@
 #include "mpi_MPI.h"
 #include "mpiJava.h"
 
-ompi_java_globals_t ompi_java;
+ompi_java_globals_t ompi_java = {0};
 int ompi_mpi_java_eager = 65536;
-opal_free_list_t ompi_java_buffers;
+opal_free_list_t ompi_java_buffers = {{{0}}};
 static void *libmpi = NULL;
 
 static void bufferConstructor(ompi_java_buffer_t *item)
@@ -128,9 +131,33 @@ OBJ_CLASS_INSTANCE(ompi_java_buffer_t,
  */
 jint JNI_OnLoad(JavaVM *vm, void *reserved)
 {
+    // Ensure that PSM signal hijacking is disabled *before* loading
+    // the library (see comment in the function for more detail).
+    opal_init_psm();
+
     libmpi = dlopen("libmpi." OPAL_DYN_LIB_SUFFIX, RTLD_NOW | RTLD_GLOBAL);
 
-    if(libmpi == NULL)
+#if defined(HAVE_DL_INFO) && defined(HAVE_LIBGEN_H)
+    /*
+     * OS X El Capitan does not propagate DYLD_LIBRARY_PATH to children any more
+     * so if previous dlopen failed, try to open libmpi in the same directory
+     * than the current libmpi_java
+     */
+    if(NULL == libmpi) {
+        Dl_info info;
+        if(0 != dladdr((void *)JNI_OnLoad, &info)) {
+            char libmpipath[OPAL_PATH_MAX];
+            char *libmpijavapath = strdup(info.dli_fname);
+            if (NULL != libmpijavapath) {
+                snprintf(libmpipath, OPAL_PATH_MAX-1, "%s/libmpi." OPAL_DYN_LIB_SUFFIX, dirname(libmpijavapath));
+                free(libmpijavapath);
+                libmpi = dlopen(libmpipath, RTLD_NOW | RTLD_GLOBAL);
+            }
+        }
+    }
+#endif
+
+    if(NULL == libmpi)
     {
         fprintf(stderr, "Java bindings failed to load libmpi: %s\n",dlerror());
         exit(1);
@@ -149,8 +176,20 @@ static void initFreeList(void)
 {
     OBJ_CONSTRUCT(&ompi_java_buffers, opal_free_list_t);
 
-    int r = opal_free_list_init(&ompi_java_buffers, sizeof(ompi_java_buffer_t),
-                                OBJ_CLASS(ompi_java_buffer_t), 2, -1, 2);
+    int r = opal_free_list_init(&ompi_java_buffers,
+                                sizeof(ompi_java_buffer_t),
+                                opal_cache_line_size,
+                                OBJ_CLASS(ompi_java_buffer_t),
+                                0, /* payload size */
+                                0, /* payload align */
+                                2, /* initial elements to alloc */
+                                -1, /* max elements */
+                                2, /* num elements per alloc */
+                                NULL, /* mpool */
+                                0, /* mpool reg flags */
+                                NULL, /* unused0 */
+                                NULL, /* item_init */
+                                NULL /* inem_init context */);
     if(r != OPAL_SUCCESS)
     {
         fprintf(stderr, "Unable to initialize ompi_java_buffers.\n");
@@ -196,6 +235,8 @@ static void deleteClasses(JNIEnv *env)
 {
     (*env)->DeleteGlobalRef(env, ompi_java.CartParmsClass);
     (*env)->DeleteGlobalRef(env, ompi_java.ShiftParmsClass);
+    (*env)->DeleteGlobalRef(env, ompi_java.VersionClass);
+    (*env)->DeleteGlobalRef(env, ompi_java.CountClass);
     (*env)->DeleteGlobalRef(env, ompi_java.GraphParmsClass);
     (*env)->DeleteGlobalRef(env, ompi_java.DistGraphNeighborsClass);
     (*env)->DeleteGlobalRef(env, ompi_java.StatusClass);
@@ -249,6 +290,12 @@ JNIEXPORT jobject JNICALL Java_mpi_MPI_newDoubleInt(JNIEnv *env, jclass clazz)
     return (*env)->NewObject(env, c, m, iOff, sizeof(int));
 }
 
+JNIEXPORT void JNICALL Java_mpi_MPI_initVersion(JNIEnv *env, jclass jthis)
+{
+    ompi_java.VersionClass = findClass(env, "mpi/Version");
+    ompi_java.VersionInit = (*env)->GetMethodID(env, ompi_java.VersionClass, "<init>", "(II)V");
+}
+
 JNIEXPORT jobjectArray JNICALL Java_mpi_MPI_Init_1jni(
         JNIEnv *env, jclass clazz, jobjectArray argv)
 {
@@ -345,6 +392,26 @@ JNIEXPORT void JNICALL Java_mpi_MPI_Finalize_1jni(JNIEnv *env, jclass obj)
     deleteClasses(env);
 }
 
+JNIEXPORT jobject JNICALL Java_mpi_MPI_getVersionJNI(JNIEnv *env, jclass jthis)
+{
+	int version, subversion;
+	int rc = MPI_Get_version(&version, &subversion);
+	ompi_java_exceptionCheck(env, rc);
+
+	return (*env)->NewObject(env, ompi_java.VersionClass,
+	                             ompi_java.VersionInit, version, subversion);
+}
+
+JNIEXPORT jstring JNICALL Java_mpi_MPI_getLibVersionJNI(JNIEnv *env, jclass jthis)
+{
+	int length;
+	char version[MPI_MAX_LIBRARY_VERSION_STRING];
+	int rc = MPI_Get_library_version(version, &length);
+	ompi_java_exceptionCheck(env, rc);
+
+	return (*env)->NewStringUTF(env, version);
+}
+
 JNIEXPORT jint JNICALL Java_mpi_MPI_getProcessorName(
                        JNIEnv *env, jclass obj, jbyteArray buf)
 {
@@ -506,8 +573,6 @@ static void setArrayRegion(JNIEnv *env, jobject buf, int baseType,
 
 static void* getBuffer(JNIEnv *env, ompi_java_buffer_t **item, int size)
 {
-    int rc;
-
     if(size > ompi_mpi_java_eager)
     {
         *item = NULL;
@@ -516,11 +581,8 @@ static void* getBuffer(JNIEnv *env, ompi_java_buffer_t **item, int size)
     else
     {
         opal_free_list_item_t *freeListItem;
-        OPAL_FREE_LIST_GET(&ompi_java_buffers, freeListItem, rc);
-        if (OPAL_SUCCESS != rc) {
-            return NULL;
-        }
-        
+        freeListItem = opal_free_list_get (&ompi_java_buffers);
+
         ompi_java_exceptionCheck(env, NULL == freeListItem ? MPI_ERR_NO_MEM :
                                  MPI_SUCCESS);
         if (NULL == freeListItem) {
@@ -541,7 +603,7 @@ static void releaseBuffer(void *ptr, ompi_java_buffer_t *item)
     else
     {
         assert(item->buffer == ptr);
-        OPAL_FREE_LIST_RETURN(&ompi_java_buffers, (opal_free_list_item_t*)item);
+        opal_free_list_return (&ompi_java_buffers, (opal_free_list_item_t*)item);
     }
 }
 
@@ -972,6 +1034,30 @@ void ompi_java_forgetIntArray(JNIEnv *env, jintArray array,
     (*env)->ReleaseIntArrayElements(env, array, jptr, JNI_ABORT);
 }
 
+void ompi_java_getDatatypeArray(JNIEnv *env, jlongArray array,
+                           jlong **jptr, MPI_Datatype **cptr)
+{
+    jlong *jLongs = (*env)->GetLongArrayElements(env, array, NULL);
+    *jptr = jLongs;
+
+    int i, length = (*env)->GetArrayLength(env, array);
+    MPI_Datatype *cDatatypes = calloc(length, sizeof(MPI_Datatype));
+
+    for(i = 0; i < length; i++){
+        cDatatypes[i] = (MPI_Datatype)jLongs[i];
+    }
+    *cptr = cDatatypes;
+}
+
+void ompi_java_forgetDatatypeArray(JNIEnv *env, jlongArray array,
+                              jlong *jptr, MPI_Datatype *cptr)
+{
+    if((long)jptr != (long)cptr)
+        free(cptr);
+
+    (*env)->ReleaseLongArrayElements(env, array, jptr, JNI_ABORT);
+}
+
 void ompi_java_getBooleanArray(JNIEnv *env, jbooleanArray array,
                                jboolean **jptr, int **cptr)
 {
@@ -981,7 +1067,7 @@ void ompi_java_getBooleanArray(JNIEnv *env, jbooleanArray array,
 
     for(i = 0; i < length; i++)
         cb[i] = jb[i];
-    
+
     *jptr = jb;
     *cptr = cb;
 }
@@ -1042,6 +1128,8 @@ void ompi_java_releasePtrArray(JNIEnv *env, jlongArray array,
 
 jboolean ompi_java_exceptionCheck(JNIEnv *env, int rc)
 {
+    jboolean jni_exception;
+
     if (rc < 0) {
         /* handle ompi error code */
         rc = ompi_errcode_get_mpi_code (rc);
@@ -1049,16 +1137,13 @@ jboolean ompi_java_exceptionCheck(JNIEnv *env, int rc)
          * all Open MPI MPI error codes should be > 0. */
         assert (rc >= 0);
     }
+    jni_exception = (*env)->ExceptionCheck(env);
 
-    if(MPI_SUCCESS == rc)
+    if(MPI_SUCCESS == rc && JNI_FALSE == jni_exception)
     {
         return JNI_FALSE;
     }
-    else if((*env)->ExceptionCheck(env))
-    {
-        return JNI_TRUE;
-    }
-    else
+    else if(MPI_SUCCESS != rc)
     {
         int     errClass = ompi_mpi_errcode_get_class(rc);
         char    *message = ompi_mpi_errnum_get_string(rc);
@@ -1072,6 +1157,8 @@ jboolean ompi_java_exceptionCheck(JNIEnv *env, int rc)
         (*env)->DeleteLocalRef(env, jmessage);
         return JNI_TRUE;
     }
+    /* If we get here, a JNI error has occurred. */
+    return JNI_TRUE;
 }
 
 void* ompi_java_attrSet(JNIEnv *env, jbyteArray jval)
@@ -1082,7 +1169,7 @@ void* ompi_java_attrSet(JNIEnv *env, jbyteArray jval)
 
     (*env)->GetByteArrayRegion(env, jval,
             0, length, (jbyte*)cval + sizeof(int));
-    
+
     return cval;
 }
 
