@@ -6,17 +6,17 @@
  * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2008-2011 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2012-2013 Los Alamos National Security, LLC. All rights
+ * Copyright (c) 2012-2015 Los Alamos National Security, LLC. All rights
  *                         reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -82,6 +82,10 @@ typedef int (*mca_base_var_enum_dump_fn_t)(mca_base_var_enum_t *self, char **out
  *
  * @retval OPAL_SUCCESS on success
  * @retval OPAL_ERR_VALUE_OUT_OF_BOUNDS if not found
+ *
+ * @long This function returns the string value for a given interger value in the
+ * {string_value} parameter. The {string_value} parameter may be NULL in which case
+ * no string is returned.
  */
 typedef int (*mca_base_var_enum_sfv_fn_t)(mca_base_var_enum_t *self, const int value,
                                           const char **string_value);
@@ -102,6 +106,9 @@ typedef struct mca_base_var_enum_value_t mca_base_var_enum_value_t;
 struct mca_base_var_enum_t {
     opal_object_t super;
 
+    /** Is the enumerator statically allocated */
+    bool enum_is_static;
+
     /** Name of this enumerator. This value is duplicated from the argument provided to
         mca_base_var_enum_create() */
     char *enum_name;
@@ -162,4 +169,19 @@ OPAL_DECLSPEC OBJ_CLASS_DECLARATION(mca_base_var_enum_t);
 OPAL_DECLSPEC int mca_base_var_enum_create (const char *name, const mca_base_var_enum_value_t values[],
                                             mca_base_var_enum_t **enumerator);
 
+/* standard enumerators. it is invalid to call OBJ_RELEASE on any of these enumerators */
+/**
+ * Boolean enumerator
+ *
+ * This enumerator maps:
+ *   positive integer, true, yes, enabled, t -> 1
+ *   0, false, no, disabled, f -> 0
+ */
+extern mca_base_var_enum_t mca_base_var_enum_bool;
+
+/**
+ * Verbosity level enumerator
+ */
+extern mca_base_var_enum_t mca_base_var_enum_verbose;
+
 #endif /* !defined(MCA_BASE_VAR_ENUM_H) */
