@@ -10,8 +10,9 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2011      Los Alamos National Security, LLC.
+ * Copyright (c) 2011-2015 Los Alamos National Security, LLC.
  *                         All rights reserved.
+ * Copyright (c) 2014-2015 Intel, Inc.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -27,16 +28,16 @@
  */
 #include "orte_config.h"
 
-#ifdef HAVE_STRING_H
 #include <string.h>
-#endif
 #ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif
 
-#include "opal/mca/mca.h"
+#include "orte/mca/mca.h"
 #include "opal/dss/dss.h"
 #include "opal/threads/threads.h"
+#include "opal/util/argv.h"
+#include "opal/util/opal_environ.h"
 
 #include "orte/constants.h"
 #include "orte/types.h"
@@ -65,11 +66,11 @@ int orte_plm_base_comm_start(void)
     if (recv_issued) {
         return ORTE_SUCCESS;
     }
-    
+
     OPAL_OUTPUT_VERBOSE((5, orte_plm_base_framework.framework_output,
                          "%s plm:base:receive start comm",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
-    
+
     orte_rml.recv_buffer_nb(ORTE_NAME_WILDCARD,
                             ORTE_RML_TAG_PLM,
                             ORTE_RML_PERSISTENT,
@@ -86,7 +87,7 @@ int orte_plm_base_comm_start(void)
                                 orte_plm_base_daemon_failed, NULL);
     }
     recv_issued = true;
-    
+
     return ORTE_SUCCESS;
 }
 
@@ -96,17 +97,17 @@ int orte_plm_base_comm_stop(void)
     if (!recv_issued) {
         return ORTE_SUCCESS;
     }
-    
+
     OPAL_OUTPUT_VERBOSE((5, orte_plm_base_framework.framework_output,
                          "%s plm:base:receive stop comm",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
-    
+
     orte_rml.recv_cancel(ORTE_NAME_WILDCARD, ORTE_RML_TAG_PLM);
     if (ORTE_PROC_IS_HNP) {
         orte_rml.recv_cancel(ORTE_NAME_WILDCARD, ORTE_RML_TAG_ORTED_CALLBACK);
     }
     recv_issued = false;
-    
+
     return ORTE_SUCCESS;
 }
 
@@ -130,9 +131,9 @@ void orte_plm_base_recv(int status, orte_process_name_t* sender,
     orte_process_name_t name;
     pid_t pid;
     bool running;
-    int8_t flag;
-    int i;
+    int i, room;
     char **env;
+    char *prefix_dir;
 
     OPAL_OUTPUT_VERBOSE((5, orte_plm_base_framework.framework_output,
                          "%s plm:base:receive processing msg",
@@ -143,42 +144,44 @@ void orte_plm_base_recv(int status, orte_process_name_t* sender,
         ORTE_ERROR_LOG(rc);
         goto CLEANUP;
     }
-        
+
     switch (command) {
     case ORTE_PLM_LAUNCH_JOB_CMD:
         OPAL_OUTPUT_VERBOSE((5, orte_plm_base_framework.framework_output,
                              "%s plm:base:receive job launch command from %s",
                              ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                              ORTE_NAME_PRINT(sender)));
-                
+
         /* unpack the job object */
         count = 1;
         if (ORTE_SUCCESS != (rc = opal_dss.unpack(buffer, &jdata, &count, ORTE_JOB))) {
             ORTE_ERROR_LOG(rc);
             goto ANSWER_LAUNCH;
         }
-            
+
         /* record the sender so we know who to respond to */
         jdata->originator.jobid = sender->jobid;
         jdata->originator.vpid = sender->vpid;
 
         /* get the parent's job object */
-        if (NULL == (parent = orte_get_job_data_object(sender->jobid))) {
-            ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
-            goto ANSWER_LAUNCH;
-        }
-       /* if the prefix was set in the parent's job, we need to transfer
-         * that prefix to the child's app_context so any further launch of
-         * orteds can find the correct binary. There always has to be at
-         * least one app_context in both parent and child, so we don't
-         * need to check that here. However, be sure not to overwrite
-         * the prefix if the user already provided it!
-         */
-        app = (orte_app_context_t*)opal_pointer_array_get_item(parent->apps, 0);
-        child_app = (orte_app_context_t*)opal_pointer_array_get_item(jdata->apps, 0);
-        if (NULL != app->prefix_dir &&
-            NULL == child_app->prefix_dir) {
-            child_app->prefix_dir = strdup(app->prefix_dir);
+        if (NULL != (parent = orte_get_job_data_object(sender->jobid))) {
+            /* if the prefix was set in the parent's job, we need to transfer
+             * that prefix to the child's app_context so any further launch of
+             * orteds can find the correct binary. There always has to be at
+             * least one app_context in both parent and child, so we don't
+             * need to check that here. However, be sure not to overwrite
+             * the prefix if the user already provided it!
+             */
+            app = (orte_app_context_t*)opal_pointer_array_get_item(parent->apps, 0);
+            child_app = (orte_app_context_t*)opal_pointer_array_get_item(jdata->apps, 0);
+            prefix_dir = NULL;
+            if (orte_get_attribute(&app->attributes, ORTE_APP_PREFIX_DIR, (void**)&prefix_dir, OPAL_STRING) &&
+                !orte_get_attribute(&child_app->attributes, ORTE_APP_PREFIX_DIR, NULL, OPAL_STRING)) {
+                orte_set_attribute(&child_app->attributes, ORTE_APP_PREFIX_DIR, ORTE_ATTR_GLOBAL, prefix_dir, OPAL_STRING);
+            }
+            if (NULL != prefix_dir) {
+                free(prefix_dir);
+            }
         }
 
         /* if the user asked to forward any envars, cycle through the app contexts
@@ -205,21 +208,23 @@ void orte_plm_base_recv(int status, orte_process_name_t* sender,
             goto ANSWER_LAUNCH;
         }
 
-        if( NULL == parent->bookmark ) {
-            /* find the sender's node in the job map */
-            if (NULL != (proc = (orte_proc_t*)opal_pointer_array_get_item(parent->procs, sender->vpid))) {
-                /* set the bookmark so the child starts from that place - this means
-                 * that the first child process could be co-located with the proc
-                 * that called comm_spawn, assuming slots remain on that node. Otherwise,
-                 * the procs will start on the next available node
-                 */
-                jdata->bookmark = proc->node;
+        if (NULL != parent) {
+            if (NULL == parent->bookmark) {
+                /* find the sender's node in the job map */
+                if (NULL != (proc = (orte_proc_t*)opal_pointer_array_get_item(parent->procs, sender->vpid))) {
+                    /* set the bookmark so the child starts from that place - this means
+                     * that the first child process could be co-located with the proc
+                     * that called comm_spawn, assuming slots remain on that node. Otherwise,
+                     * the procs will start on the next available node
+                     */
+                    jdata->bookmark = proc->node;
+                }
+            } else {
+                jdata->bookmark = parent->bookmark;
             }
-        } else {
-            jdata->bookmark = parent->bookmark;
+            /* provide the parent's last object */
+            jdata->bkmark_obj = parent->bkmark_obj;
         }
-        /* provide the parent's last object */
-        jdata->bkmark_obj = parent->bkmark_obj;
 
         /* launch it */
         OPAL_OUTPUT_VERBOSE((5, orte_plm_base_framework.framework_output,
@@ -237,20 +242,32 @@ void orte_plm_base_recv(int status, orte_process_name_t* sender,
 
         /* setup the response */
         answer = OBJ_NEW(opal_buffer_t);
-        
+
         /* pack the error code to be returned */
         if (ORTE_SUCCESS != (ret = opal_dss.pack(answer, &rc, 1, OPAL_INT32))) {
             ORTE_ERROR_LOG(ret);
         }
-                
+
+        /* pack an invalid jobid */
+        job = ORTE_JOBID_INVALID;
+        if (ORTE_SUCCESS != (ret = opal_dss.pack(answer, &job, 1, ORTE_JOBID))) {
+            ORTE_ERROR_LOG(ret);
+        }
+        /* pack the room number of the request */
+        if (orte_get_attribute(&jdata->attributes, ORTE_JOB_ROOM_NUM, (void**)&room, OPAL_INT)) {
+            if (ORTE_SUCCESS != (ret = opal_dss.pack(answer, &room, 1, OPAL_INT))) {
+                ORTE_ERROR_LOG(ret);
+            }
+        }
+
         /* send the response back to the sender */
-        if (0 > (ret = orte_rml.send_buffer_nb(sender, answer, ORTE_RML_TAG_PLM_PROXY,
+        if (0 > (ret = orte_rml.send_buffer_nb(sender, answer, ORTE_RML_TAG_LAUNCH_RESP,
                                                orte_rml_send_callback, NULL))) {
             ORTE_ERROR_LOG(ret);
             OBJ_RELEASE(answer);
         }
         break;
-                
+
     case ORTE_PLM_UPDATE_PROC_STATE:
         opal_output_verbose(5, orte_plm_base_framework.framework_output,
                             "%s plm:base:receive update proc state command from %s",
@@ -258,12 +275,12 @@ void orte_plm_base_recv(int status, orte_process_name_t* sender,
                             ORTE_NAME_PRINT(sender));
         count = 1;
         while (ORTE_SUCCESS == (rc = opal_dss.unpack(buffer, &job, &count, ORTE_JOBID))) {
-                    
+
             opal_output_verbose(5, orte_plm_base_framework.framework_output,
                                 "%s plm:base:receive got update_proc_state for job %s",
                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                                 ORTE_JOBID_PRINT(job));
-                    
+
             name.jobid = job;
             running = false;
             /* get the job object */
@@ -296,7 +313,7 @@ void orte_plm_base_recv(int status, orte_process_name_t* sender,
                     ORTE_ERROR_LOG(rc);
                     goto CLEANUP;
                 }
-                        
+
                 OPAL_OUTPUT_VERBOSE((5, orte_plm_base_framework.framework_output,
                                      "%s plm:base:receive got update_proc_state for vpid %lu state %s exit_code %d",
                                      ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
@@ -335,8 +352,8 @@ void orte_plm_base_recv(int status, orte_process_name_t* sender,
             rc = ORTE_SUCCESS;
         }
         break;
-                
-    case ORTE_PLM_INIT_ROUTES_CMD:
+
+    case ORTE_PLM_REGISTERED_CMD:
         count=1;
         if (ORTE_SUCCESS != (rc = opal_dss.unpack(buffer, &job, &count, ORTE_JOBID))) {
             ORTE_ERROR_LOG(rc);
@@ -351,37 +368,10 @@ void orte_plm_base_recv(int status, orte_process_name_t* sender,
         }
         count=1;
         while (ORTE_SUCCESS == opal_dss.unpack(buffer, &vpid, &count, ORTE_VPID)) {
-            if (ORTE_VPID_INVALID == vpid) {
-                break;
-            }
             name.vpid = vpid;
-            /* unpack the mpi proc flag */
-            count=1;
-            if (OPAL_SUCCESS != (rc = opal_dss.unpack(buffer, &flag, &count, OPAL_INT8))) {
-                ORTE_ERROR_LOG(rc);
-                goto DEPART;
-            }
-            /* get the proc data object */
-            if (NULL == (proc = (orte_proc_t*)opal_pointer_array_get_item(jdata->procs, vpid))) {
-                ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
-                rc = ORTE_ERR_NOT_FOUND;
-                goto DEPART;
-            }
-            if (1 == flag) {
-                proc->mpi_proc = true;
-            } else {
-                proc->mpi_proc = false;
-            }
             ORTE_ACTIVATE_PROC_STATE(&name, ORTE_PROC_STATE_REGISTERED);
             count=1;
         }
-        /* pass the remainder of the buffer to the active module's
-         * init_routes API
-         */
-        if (ORTE_SUCCESS != (rc = orte_routed.init_routes(job, buffer))) {
-            ORTE_ERROR_LOG(rc);
-            ORTE_FORCED_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
-        }
         break;
 
     default:
@@ -389,19 +379,19 @@ void orte_plm_base_recv(int status, orte_process_name_t* sender,
         rc = ORTE_ERR_VALUE_OUT_OF_BOUNDS;
         break;
     }
-        
+
  CLEANUP:
     if (ORTE_SUCCESS != rc) {
         goto DEPART;
     }
-        
+
  DEPART:
     /* see if an error occurred - if so, wakeup the HNP so we can exit */
     if (ORTE_PROC_IS_HNP && ORTE_SUCCESS != rc) {
         jdata = NULL;
         ORTE_FORCED_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
     }
-    
+
     OPAL_OUTPUT_VERBOSE((5, orte_plm_base_framework.framework_output,
                          "%s plm:base:receive done processing commands",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
