@@ -5,7 +5,7 @@
  * Copyright (c) 2004-2011 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
@@ -13,11 +13,13 @@
  * Copyright (c) 2009-2010 Oracle and/or its affiliates.  All rights reserved.
  * Copyright (c) 2011-2013 Los Alamos National Security, LLC.
  *                         All rights reserved.
- * Copyright (c) 2013-2014 Intel, Inc. All rights reserved
+ * Copyright (c) 2013-2015 Intel, Inc. All rights reserved
+ * Copyright (c) 2014-2015 Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -29,8 +31,8 @@
 #include <sys/time.h>
 #endif
 
-#include "opal/mca/db/db.h"
 #include "opal/mca/hwloc/hwloc.h"
+#include "opal/mca/pmix/pmix.h"
 #include "opal/util/argv.h"
 #include "opal/util/output.h"
 #include "opal/class/opal_hash_table.h"
@@ -52,18 +54,15 @@
 #include "orte/runtime/data_type_support/orte_dt_support.h"
 
 /* State Machine */
-opal_list_t orte_job_states;
-opal_list_t orte_proc_states;
+opal_list_t orte_job_states = {{0}};
+opal_list_t orte_proc_states = {{0}};
 
 /* a clean output channel without prefix */
 int orte_clean_output = -1;
 
 /* globals used by RTE */
-bool orte_timing;
-FILE *orte_timing_output = NULL;
-bool orte_timing_details;
 bool orte_debug_daemons_file_flag = false;
-bool orte_leave_session_attached;
+bool orte_leave_session_attached = false;
 bool orte_do_not_launch = false;
 bool orted_spin_flag = false;
 char *orte_local_cpu_type = NULL;
@@ -71,6 +70,7 @@ char *orte_local_cpu_model = NULL;
 char *orte_basename = NULL;
 bool orte_coprocessors_detected = false;
 opal_hash_table_t *orte_coprocessors = NULL;
+char *orte_topo_signature = NULL;
 
 /* ORTE OOB port flags */
 bool orte_static_ports = false;
@@ -79,13 +79,12 @@ bool orte_standalone_operation = false;
 
 bool orte_keep_fqdn_hostnames = false;
 bool orte_have_fqdn_allocation = false;
-bool orte_show_resolved_nodenames;
-bool orte_retain_aliases;
-int orte_use_hostname_alias;
+bool orte_show_resolved_nodenames = false;
+bool orte_retain_aliases = false;
+int orte_use_hostname_alias = -1;
 
-int orted_debug_failure;
-int orted_debug_failure_delay;
-bool orte_homogeneous_nodes = false;
+int orted_debug_failure = -1;
+int orted_debug_failure_delay = -1;
 bool orte_hetero_apps = false;
 bool orte_hetero_nodes = false;
 bool orte_never_launched = false;
@@ -93,15 +92,16 @@ bool orte_devel_level_output = false;
 bool orte_display_topo_with_map = false;
 bool orte_display_diffable_output = false;
 
-char **orte_launch_environ;
+char **orte_launch_environ = NULL;
 
 bool orte_hnp_is_allocated = false;
-bool orte_allocation_required;
+bool orte_allocation_required = false;
 bool orte_managed_allocation = false;
 char *orte_set_slots = NULL;
-bool orte_display_allocation;
-bool orte_display_devel_allocation;
+bool orte_display_allocation = false;
+bool orte_display_devel_allocation = false;
 bool orte_soft_locations = false;
+int orted_pmi_version = 0;
 
 /* launch agents */
 char *orte_launch_agent = NULL;
@@ -109,10 +109,10 @@ char **orted_cmd_line=NULL;
 char **orte_fork_agent=NULL;
 
 /* debugger job */
-bool orte_debugger_dump_proctable;
-char *orte_debugger_test_daemon;
-bool orte_debugger_test_attach;
-int orte_debugger_check_rate;
+bool orte_debugger_dump_proctable = false;
+char *orte_debugger_test_daemon = NULL;
+bool orte_debugger_test_attach = false;
+int orte_debugger_check_rate = -1;
 
 /* exit flags */
 int orte_exit_status = 0;
@@ -122,29 +122,29 @@ bool orte_job_term_ordered = false;
 bool orte_orteds_term_ordered = false;
 bool orte_allowed_exit_without_sync = false;
 
-int orte_startup_timeout;
-int orte_timeout_usec_per_proc;
-float orte_max_timeout;
+int orte_startup_timeout = -1;
+int orte_timeout_usec_per_proc = -1;
+float orte_max_timeout = -1.0;
 orte_timer_t *orte_mpiexec_timeout = NULL;
 
 opal_buffer_t *orte_tree_launch_cmd = NULL;
 
 /* global arrays for data storage */
-opal_pointer_array_t *orte_job_data;
-opal_pointer_array_t *orte_node_pool;
-opal_pointer_array_t *orte_node_topologies;
-opal_pointer_array_t *orte_local_children;
+opal_pointer_array_t *orte_job_data = NULL;
+opal_pointer_array_t *orte_node_pool = NULL;
+opal_pointer_array_t *orte_node_topologies = NULL;
+opal_pointer_array_t *orte_local_children = NULL;
 orte_vpid_t orte_total_procs = 0;
 
 /* IOF controls */
-bool orte_tag_output;
-bool orte_timestamp_output;
-char *orte_output_filename;
+bool orte_tag_output = false;
+bool orte_timestamp_output = false;
+char *orte_output_filename = NULL;
 /* generate new xterm windows to display output from specified ranks */
-char *orte_xterm;
+char *orte_xterm = NULL;
 
 /* whether or not to forward SIGTSTP and SIGCONT signals */
-bool orte_forward_job_control;
+bool orte_forward_job_control = false;
 
 /* report launch progress */
 bool orte_report_launch_progress = false;
@@ -155,6 +155,7 @@ bool orte_default_hostfile_given = false;
 char *orte_rankfile = NULL;
 int orte_num_allocated_nodes = 0;
 char *orte_node_regex = NULL;
+char *orte_default_dash_host = NULL;
 
 /* tool communication controls */
 bool orte_report_events = false;
@@ -167,19 +168,18 @@ bool orte_report_bindings = false;
 bool orte_do_not_barrier = false;
 
 /* process recovery */
-bool orte_enable_recovery;
-int32_t orte_max_restarts;
+bool orte_enable_recovery = false;
+int32_t orte_max_restarts = 0;
 
 /* exit status reporting */
-bool orte_report_child_jobs_separately;
-struct timeval orte_child_time_to_exit;
-bool orte_abort_non_zero_exit;
+bool orte_report_child_jobs_separately = false;
+struct timeval orte_child_time_to_exit = {0};
+bool orte_abort_non_zero_exit = false;
 
 /* length of stat history to keep */
-int orte_stat_history_size;
+int orte_stat_history_size = -1;
 
 /* envars to forward */
-char *orte_forward_envars = NULL;
 char **orte_forwarded_envars = NULL;
 
 /* map-reduce mode */
@@ -192,16 +192,12 @@ bool orte_map_stddiag_to_stderr = false;
 /* maximum size of virtual machine - used to subdivide allocation */
 int orte_max_vm_size = -1;
 
-/* progress thread */
-opal_thread_t orte_progress_thread;
-
-/* global nidmap/pidmap for daemons to give to apps */
-opal_byte_object_t orte_nidmap;
-opal_byte_object_t orte_pidmap;
-
 /* user debugger */
 char *orte_base_user_debugger = NULL;
 
+/* modex cutoff */
+uint32_t orte_direct_modex_cutoff = UINT32_MAX;
+
 int orte_debug_output = -1;
 bool orte_debug_daemons_flag = false;
 bool orte_xml_output = false;
@@ -223,7 +219,7 @@ int orte_dt_init(void)
 
     /* set default output */
     orte_debug_output = opal_output_open(NULL);
-    
+
     /* open up the verbose output for ORTE debugging */
     if (orte_debug_flag || 0 < orte_debug_verbosity ||
         (orte_debug_daemons_flag && (ORTE_PROC_IS_DAEMON || ORTE_PROC_IS_HNP))) {
@@ -246,41 +242,6 @@ int orte_dt_init(void)
         ORTE_ERROR_LOG(rc);
         return rc;
     }
-    tmp = ORTE_NAME;
-    if (ORTE_SUCCESS != (rc = opal_dss.register_type(orte_dt_pack_name,
-                                                     orte_dt_unpack_name,
-                                                     (opal_dss_copy_fn_t)orte_dt_copy_name,
-                                                     (opal_dss_compare_fn_t)orte_dt_compare_name,
-                                                     (opal_dss_print_fn_t)orte_dt_print_name,
-                                                     OPAL_DSS_UNSTRUCTURED,
-                                                     "ORTE_NAME", &tmp))) {
-        ORTE_ERROR_LOG(rc);
-        return rc;
-    }
-    
-    tmp = ORTE_VPID;
-    if (ORTE_SUCCESS != (rc = opal_dss.register_type(orte_dt_pack_vpid,
-                                                     orte_dt_unpack_vpid,
-                                                     (opal_dss_copy_fn_t)orte_dt_copy_vpid,
-                                                     (opal_dss_compare_fn_t)orte_dt_compare_vpid,
-                                                     (opal_dss_print_fn_t)orte_dt_std_print,
-                                                     OPAL_DSS_UNSTRUCTURED,
-                                                     "ORTE_VPID", &tmp))) {
-        ORTE_ERROR_LOG(rc);
-        return rc;
-    }
-    
-    tmp = ORTE_JOBID;
-    if (ORTE_SUCCESS != (rc = opal_dss.register_type(orte_dt_pack_jobid,
-                                                     orte_dt_unpack_jobid,
-                                                     (opal_dss_copy_fn_t)orte_dt_copy_jobid,
-                                                     (opal_dss_compare_fn_t)orte_dt_compare_jobid,
-                                                     (opal_dss_print_fn_t)orte_dt_std_print,
-                                                     OPAL_DSS_UNSTRUCTURED,
-                                                     "ORTE_JOBID", &tmp))) {
-        ORTE_ERROR_LOG(rc);
-        return rc;
-    }
 
     tmp = ORTE_JOB;
     if (ORTE_SUCCESS != (rc = opal_dss.register_type(orte_dt_pack_job,
@@ -293,7 +254,7 @@ int orte_dt_init(void)
         ORTE_ERROR_LOG(rc);
         return rc;
     }
-    
+
     tmp = ORTE_NODE;
     if (ORTE_SUCCESS != (rc = opal_dss.register_type(orte_dt_pack_node,
                                                      orte_dt_unpack_node,
@@ -305,7 +266,7 @@ int orte_dt_init(void)
         ORTE_ERROR_LOG(rc);
         return rc;
     }
-    
+
     tmp = ORTE_PROC;
     if (ORTE_SUCCESS != (rc = opal_dss.register_type(orte_dt_pack_proc,
                                                      orte_dt_unpack_proc,
@@ -317,7 +278,7 @@ int orte_dt_init(void)
         ORTE_ERROR_LOG(rc);
         return rc;
     }
-    
+
     tmp = ORTE_APP_CONTEXT;
     if (ORTE_SUCCESS != (rc = opal_dss.register_type(orte_dt_pack_app_context,
                                                      orte_dt_unpack_app_context,
@@ -329,7 +290,7 @@ int orte_dt_init(void)
         ORTE_ERROR_LOG(rc);
         return rc;
     }
-    
+
     tmp = ORTE_NODE_STATE;
     if (ORTE_SUCCESS != (rc = opal_dss.register_type(orte_dt_pack_node_state,
                                                      orte_dt_unpack_node_state,
@@ -341,7 +302,7 @@ int orte_dt_init(void)
         ORTE_ERROR_LOG(rc);
         return rc;
     }
-    
+
     tmp = ORTE_PROC_STATE;
     if (ORTE_SUCCESS != (rc = opal_dss.register_type(orte_dt_pack_proc_state,
                                                      orte_dt_unpack_proc_state,
@@ -353,7 +314,7 @@ int orte_dt_init(void)
         ORTE_ERROR_LOG(rc);
         return rc;
     }
-    
+
     tmp = ORTE_JOB_STATE;
     if (ORTE_SUCCESS != (rc = opal_dss.register_type(orte_dt_pack_job_state,
                                                      orte_dt_unpack_job_state,
@@ -365,7 +326,7 @@ int orte_dt_init(void)
         ORTE_ERROR_LOG(rc);
         return rc;
     }
-    
+
     tmp = ORTE_EXIT_CODE;
     if (ORTE_SUCCESS != (rc = opal_dss.register_type(orte_dt_pack_exit_code,
                                                      orte_dt_unpack_exit_code,
@@ -389,7 +350,7 @@ int orte_dt_init(void)
         ORTE_ERROR_LOG(rc);
         return rc;
     }
-    
+
     tmp = ORTE_RML_TAG;
     if (ORTE_SUCCESS != (rc = opal_dss.register_type(orte_dt_pack_tag,
                                                       orte_dt_unpack_tag,
@@ -425,19 +386,43 @@ int orte_dt_init(void)
         ORTE_ERROR_LOG(rc);
         return rc;
     }
-    
-    return ORTE_SUCCESS;    
+
+    tmp = ORTE_ATTRIBUTE;
+    if (ORTE_SUCCESS != (rc = opal_dss.register_type(orte_dt_pack_attr,
+                                                     orte_dt_unpack_attr,
+                                                     (opal_dss_copy_fn_t)orte_dt_copy_attr,
+                                                     (opal_dss_compare_fn_t)orte_dt_compare_attr,
+                                                     (opal_dss_print_fn_t)orte_dt_print_attr,
+                                                     OPAL_DSS_STRUCTURED,
+                                                     "ORTE_ATTRIBUTE", &tmp))) {
+        ORTE_ERROR_LOG(rc);
+        return rc;
+    }
+
+    tmp = ORTE_SIGNATURE;
+    if (ORTE_SUCCESS != (rc = opal_dss.register_type(orte_dt_pack_sig,
+                                                     orte_dt_unpack_sig,
+                                                     (opal_dss_copy_fn_t)orte_dt_copy_sig,
+                                                     (opal_dss_compare_fn_t)orte_dt_compare_sig,
+                                                     (opal_dss_print_fn_t)orte_dt_print_sig,
+                                                     OPAL_DSS_STRUCTURED,
+                                                     "ORTE_SIGNATURE", &tmp))) {
+        ORTE_ERROR_LOG(rc);
+        return rc;
+    }
+
+    return ORTE_SUCCESS;
 }
 
 orte_job_t* orte_get_job_data_object(orte_jobid_t job)
 {
     int32_t ljob;
-    
+
     /* if the job data wasn't setup, we cannot provide the data */
     if (NULL == orte_job_data) {
         return NULL;
     }
-    
+
     /* the job is indexed by its local jobid, so we can
      * just look it up here. it is not an error for this
      * to not be found - could just be
@@ -484,33 +469,39 @@ char* orte_get_proc_hostname(orte_process_name_t *proc)
     char *hostname;
     int rc;
 
+    /* if we are a tool, then we have no way of obtaining
+     * this info */
+    if (ORTE_PROC_IS_TOOL) {
+        return NULL;
+    }
+
+    /* don't bother error logging any not-found situations
+     * as the layer above us will have something to say
+     * about it */
     if (ORTE_PROC_IS_DAEMON || ORTE_PROC_IS_HNP) {
         /* look it up on our arrays */
         if (NULL == (proct = orte_get_proc_object(proc))) {
-            ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
             return NULL;
         }
         if (NULL == proct->node || NULL == proct->node->name) {
-            ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
             return NULL;
         }
         return proct->node->name;
     }
 
-    /* if we are an app, get the pointer from the modex db */
-    if (ORTE_SUCCESS != (rc = opal_db.fetch_pointer((opal_identifier_t*)proc,
-                                                    ORTE_DB_HOSTNAME,
-                                                    (void**)&hostname, OPAL_STRING))) {
-        ORTE_ERROR_LOG(rc);
-        return NULL;
-    }
+    /* if we are an app, get the data from the modex db */
+    OPAL_MODEX_RECV_VALUE(rc, OPAL_PMIX_HOSTNAME,
+                          (opal_process_name_t*)proc,
+                          &hostname, OPAL_STRING);
+
+    /* user is responsible for releasing the data */
     return hostname;
 }
 
 orte_node_rank_t orte_get_proc_node_rank(orte_process_name_t *proc)
 {
     orte_proc_t *proct;
-    orte_node_rank_t noderank, *nr;
+    orte_node_rank_t *noderank, nd;
     int rc;
 
     if (ORTE_PROC_IS_DAEMON || ORTE_PROC_IS_HNP) {
@@ -523,14 +514,13 @@ orte_node_rank_t orte_get_proc_node_rank(orte_process_name_t *proc)
     }
 
     /* if we are an app, get the value from the modex db */
-    nr = &noderank;
-    if (ORTE_SUCCESS != (rc = opal_db.fetch_pointer((opal_identifier_t*)proc,
-                                                    ORTE_DB_NODERANK,
-                                                    (void**)&nr, ORTE_NODE_RANK))) {
-        ORTE_ERROR_LOG(rc);
-        return ORTE_NODE_RANK_INVALID;
-    }
-    return noderank;
+    OPAL_MODEX_RECV_VALUE(rc, OPAL_PMIX_NODE_RANK,
+                          (opal_process_name_t*)proc,
+                          &noderank, ORTE_NODE_RANK);
+
+    nd = *noderank;
+    free(noderank);
+    return nd;
 }
 
 orte_vpid_t orte_get_lowest_vpid_alive(orte_jobid_t job)
@@ -565,6 +555,7 @@ orte_vpid_t orte_get_lowest_vpid_alive(orte_jobid_t job)
     return ORTE_VPID_INVALID;
 }
 
+
 /*
  * CONSTRUCTORS, DESTRUCTORS, AND CLASS INSTANTIATIONS
  * FOR ORTE CLASSES
@@ -585,31 +576,12 @@ static void orte_app_context_construct(orte_app_context_t* app_context)
     app_context->argv=NULL;
     app_context->env=NULL;
     app_context->cwd=NULL;
-    app_context->user_specified_cwd=false;
-    app_context->set_cwd_to_session_dir = false;
-    app_context->hostfile=NULL;
-    app_context->add_hostfile=NULL;
-    app_context->add_host = NULL;
-    app_context->dash_host = NULL;
-    OBJ_CONSTRUCT(&app_context->resource_constraints, opal_list_t);
-    app_context->prefix_dir = NULL;
-    app_context->preload_binary = false;
-    app_context->preload_files  = NULL;
-    app_context->used_on_node = false;
-
-#if OPAL_ENABLE_FT_CR == 1
-    app_context->sstore_load = NULL;
-#endif
-    app_context->recovery_defined = false;
-    app_context->max_restarts = -1000;
-    app_context->max_procs_per_node = 0;
-    app_context->mandatory = false;
-    app_context->min_number_of_nodes = -1;  /* no minimum */
+    app_context->flags = 0;
+    OBJ_CONSTRUCT(&app_context->attributes, opal_list_t);
 }
 
 static void orte_app_context_destructor(orte_app_context_t* app_context)
 {
-    opal_list_item_t *item;
     int i;
     orte_proc_t *proc;
 
@@ -617,7 +589,7 @@ static void orte_app_context_destructor(orte_app_context_t* app_context)
         free (app_context->app);
         app_context->app = NULL;
     }
-    
+
     for (i=0; i < app_context->procs.size; i++) {
         if (NULL != (proc = (orte_proc_t*)opal_pointer_array_get_item(&app_context->procs, i))) {
             OBJ_RELEASE(proc);
@@ -630,60 +602,18 @@ static void orte_app_context_destructor(orte_app_context_t* app_context)
         opal_argv_free(app_context->argv);
         app_context->argv = NULL;
     }
-    
+
     if (NULL != app_context->env) {
         opal_argv_free(app_context->env);
         app_context->env = NULL;
     }
-    
+
     if (NULL != app_context->cwd) {
         free (app_context->cwd);
         app_context->cwd = NULL;
     }
-    
-    if (NULL != app_context->hostfile) {
-        free(app_context->hostfile);
-        app_context->hostfile = NULL;
-    }
-    
-    if (NULL != app_context->add_hostfile) {
-        free(app_context->add_hostfile);
-        app_context->add_hostfile = NULL;
-    }
-    
-    if (NULL != app_context->add_host) {
-        opal_argv_free(app_context->add_host);
-        app_context->add_host = NULL;
-    }
-    
-    if (NULL != app_context->dash_host) {
-        opal_argv_free(app_context->dash_host);
-        app_context->dash_host = NULL;
-    }
-    
-    while (NULL != (item = opal_list_remove_first(&app_context->resource_constraints))) {
-        OBJ_RELEASE(item);
-    }
-    OBJ_DESTRUCT(&app_context->resource_constraints);
 
-    if (NULL != app_context->prefix_dir) {
-        free(app_context->prefix_dir);
-        app_context->prefix_dir = NULL;
-    }
-    
-    app_context->preload_binary = false;
-
-    if(NULL != app_context->preload_files) {
-        free(app_context->preload_files);
-        app_context->preload_files = NULL;
-    }
-    
-#if OPAL_ENABLE_FT_CR == 1
-    if( NULL != app_context->sstore_load ) {
-        free(app_context->sstore_load);
-        app_context->sstore_load = NULL;
-    }
-#endif
+    OPAL_LIST_DESTRUCT(&app_context->attributes);
 }
 
 OBJ_CLASS_INSTANCE(orte_app_context_t,
@@ -693,20 +623,16 @@ OBJ_CLASS_INSTANCE(orte_app_context_t,
 
 static void orte_job_construct(orte_job_t* job)
 {
+    job->personality = NULL;
     job->jobid = ORTE_JOBID_INVALID;
     job->offset = 0;
-    job->updated = true;
     job->apps = OBJ_NEW(opal_pointer_array_t);
     opal_pointer_array_init(job->apps,
                             1,
                             ORTE_GLOBAL_ARRAY_MAX_SIZE,
                             2);
     job->num_apps = 0;
-    job->controls = ORTE_JOB_CONTROL_FORWARD_OUTPUT;
-    job->failure_timer = NULL;
-    job->gang_launched = true;
-    job->stdin_target = ORTE_VPID_INVALID;
-    job->stdout_target = ORTE_JOBID_INVALID;
+    job->stdin_target = 0;
     job->total_slots_alloc = 0;
     job->num_procs = 0;
     job->procs = OBJ_NEW(opal_pointer_array_t);
@@ -716,32 +642,24 @@ static void orte_job_construct(orte_job_t* job)
                             ORTE_GLOBAL_ARRAY_BLOCK_SIZE);
     job->map = NULL;
     job->bookmark = NULL;
+    job->bkmark_obj = 0;
     job->state = ORTE_JOB_STATE_UNDEF;
-    job->restart = false;
 
     job->num_mapped = 0;
     job->num_launched = 0;
     job->num_reported = 0;
     job->num_terminated = 0;
     job->num_daemons_reported = 0;
-    job->num_non_zero_exit = 0;
-    job->abort = false;
-    job->aborted_proc = NULL;
-    
+
     job->originator.jobid = ORTE_JOBID_INVALID;
     job->originator.vpid = ORTE_VPID_INVALID;
-
-    job->recovery_defined = false;
-    job->enable_recovery = false;
     job->num_local_procs = 0;
 
-    job->file_maps = NULL;
+    job->flags = 0;
+    ORTE_FLAG_SET(job, ORTE_JOB_FLAG_GANG_LAUNCHED);
+    ORTE_FLAG_SET(job, ORTE_JOB_FLAG_FORWARD_OUTPUT);
 
-#if OPAL_ENABLE_FT_CR == 1
-    job->ckpt_state = 0;
-    job->ckpt_snapshot_ref = NULL;
-    job->ckpt_snapshot_loc = NULL;
-#endif
+    OBJ_CONSTRUCT(&job->attributes, opal_list_t);
 }
 
 static void orte_job_destruct(orte_job_t* job)
@@ -750,17 +668,21 @@ static void orte_job_destruct(orte_job_t* job)
     orte_app_context_t *app;
     orte_job_t *jdata;
     int n;
+    orte_timer_t *evtimer;
 
     if (NULL == job) {
         /* probably just a race condition - just return */
         return;
     }
-    
+
     if (orte_debug_flag) {
         opal_output(0, "%s Releasing job data for %s",
                     ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), ORTE_JOBID_PRINT(job->jobid));
     }
-    
+
+    if (NULL != job->personality) {
+        free(job->personality);
+    }
     for (n=0; n < job->apps->size; n++) {
         if (NULL == (app = (orte_app_context_t*)opal_pointer_array_get_item(job->apps, n))) {
             continue;
@@ -768,16 +690,28 @@ static void orte_job_destruct(orte_job_t* job)
         OBJ_RELEASE(app);
     }
     OBJ_RELEASE(job->apps);
-    
-    if (NULL != job->failure_timer) {
-        OBJ_RELEASE(job->failure_timer);
+
+    /* release any pointers in the attributes */
+    evtimer = NULL;
+    if (orte_get_attribute(&job->attributes, ORTE_JOB_FAILURE_TIMER_EVENT,
+                           (void**)&evtimer, OPAL_PTR)) {
+        orte_remove_attribute(&job->attributes, ORTE_JOB_FAILURE_TIMER_EVENT);
+        /* the timer is a pointer to orte_timer_t */
+        OBJ_RELEASE(evtimer);
+    }
+    proc = NULL;
+    if (orte_get_attribute(&job->attributes, ORTE_JOB_ABORTED_PROC,
+                           (void**)&proc, OPAL_PTR)) {
+        orte_remove_attribute(&job->attributes, ORTE_JOB_ABORTED_PROC);
+        /* points to an orte_proc_t */
+        OBJ_RELEASE(proc);
     }
 
     if (NULL != job->map) {
         OBJ_RELEASE(job->map);
         job->map = NULL;
     }
-    
+
     for (n=0; n < job->procs->size; n++) {
         if (NULL == (proc = (orte_proc_t*)opal_pointer_array_get_item(job->procs, n))) {
             continue;
@@ -785,24 +719,12 @@ static void orte_job_destruct(orte_job_t* job)
         OBJ_RELEASE(proc);
     }
     OBJ_RELEASE(job->procs);
-    
-    if (NULL != job->file_maps) {
-        OBJ_RELEASE(job->file_maps);
-    }
 
-#if OPAL_ENABLE_FT_CR == 1
-    if (NULL != job->ckpt_snapshot_ref) {
-        free(job->ckpt_snapshot_ref);
-        job->ckpt_snapshot_ref = NULL;
-    }
-    if (NULL != job->ckpt_snapshot_loc) {
-        free(job->ckpt_snapshot_loc);
-        job->ckpt_snapshot_loc = NULL;
-    }
-#endif
-    
+    /* release the attributes */
+    OPAL_LIST_DESTRUCT(&job->attributes);
+
     /* find the job in the global array */
-    if (NULL != orte_job_data) {
+    if (NULL != orte_job_data && ORTE_JOBID_INVALID != job->jobid) {
         for (n=0; n < orte_job_data->size; n++) {
             if (NULL == (jdata = (orte_job_t*)opal_pointer_array_get_item(orte_job_data, n))) {
                 continue;
@@ -826,14 +748,7 @@ static void orte_node_construct(orte_node_t* node)
 {
     node->index = -1;
     node->name = NULL;
-    node->alias = NULL;
-    node->serial_number = NULL;
-    node->hostid = ORTE_VPID_INVALID;
     node->daemon = NULL;
-    node->daemon_launched = false;
-    node->location_verified = false;
-    node->launch_id = -1;
-    node->mapped = false;
 
     node->num_procs = 0;
     node->procs = OBJ_NEW(opal_pointer_array_t);
@@ -842,28 +757,20 @@ static void orte_node_construct(orte_node_t* node)
                             ORTE_GLOBAL_ARRAY_MAX_SIZE,
                             ORTE_GLOBAL_ARRAY_BLOCK_SIZE);
     node->next_node_rank = 0;
-    
-    node->oversubscribed = false;
+
     node->state = ORTE_NODE_STATE_UNKNOWN;
     node->slots = 0;
-    node->slots_given = false;
     node->slots_inuse = 0;
     node->slots_max = 0;
-    
-    node->username = NULL;
-    
-#if OPAL_HAVE_HWLOC
     node->topology = NULL;
-#endif
 
-    OBJ_CONSTRUCT(&node->stats, opal_ring_buffer_t);
-    opal_ring_buffer_init(&node->stats, orte_stat_history_size);
+    node->flags = 0;
+    OBJ_CONSTRUCT(&node->attributes, opal_list_t);
 }
 
 static void orte_node_destruct(orte_node_t* node)
 {
     int i;
-    opal_node_stats_t *stats;
     orte_proc_t *proc;
 
     if (NULL != node->name) {
@@ -871,22 +778,12 @@ static void orte_node_destruct(orte_node_t* node)
         node->name = NULL;
     }
 
-    if (NULL != node->serial_number) {
-        free(node->serial_number);
-        node->serial_number = NULL;
-    }
-
-    if (NULL != node->alias) {
-        opal_argv_free(node->alias);
-        node->alias = NULL;
-    }
-    
     if (NULL != node->daemon) {
         node->daemon->node = NULL;
         OBJ_RELEASE(node->daemon);
         node->daemon = NULL;
     }
-    
+
     for (i=0; i < node->procs->size; i++) {
         if (NULL != (proc = (orte_proc_t*)opal_pointer_array_get_item(node->procs, i))) {
             opal_pointer_array_set_item(node->procs, i, NULL);
@@ -894,20 +791,11 @@ static void orte_node_destruct(orte_node_t* node)
         }
     }
     OBJ_RELEASE(node->procs);
-    
-    /* we release the topology elsewhere */
 
-    if (NULL != node->username) {
-        free(node->username);
-        node->username = NULL;
-    }
-    
     /* do NOT destroy the topology */
 
-    while (NULL != (stats = (opal_node_stats_t*)opal_ring_buffer_pop(&node->stats))) {
-        OBJ_RELEASE(stats);
-    }
-    OBJ_DESTRUCT(&node->stats);
+    /* release the attributes */
+    OPAL_LIST_DESTRUCT(&node->attributes);
 }
 
 
@@ -927,82 +815,27 @@ static void orte_proc_construct(orte_proc_t* proc)
     proc->app_rank = -1;
     proc->last_errmgr_state = ORTE_PROC_STATE_UNDEF;
     proc->state = ORTE_PROC_STATE_UNDEF;
-    proc->alive = false;
-    proc->aborted = false;
-    proc->updated = true;
     proc->app_idx = 0;
-#if OPAL_HAVE_HWLOC
-    proc->locale = NULL;
-    proc->bind_location = NULL;
-    proc->cpu_bitmap = NULL;
-#endif
     proc->node = NULL;
-    proc->local_proc = false;
-    proc->do_not_barrier = false;
-    proc->prior_node = NULL;
-    proc->nodename = NULL;
     proc->exit_code = 0;      /* Assume we won't fail unless otherwise notified */
     proc->rml_uri = NULL;
-    proc->restarts = 0;
-    proc->fast_failures = 0;
-    proc->last_failure.tv_sec = 0;
-    proc->last_failure.tv_usec = 0;
-    proc->reported = false;
-    proc->beat = 0;
-    OBJ_CONSTRUCT(&proc->stats, opal_ring_buffer_t);
-    opal_ring_buffer_init(&proc->stats, orte_stat_history_size);
-    proc->registered = false;
-    proc->mpi_proc = false;
-    proc->deregistered = false;
-    proc->iof_complete = false;
-    proc->waitpid_recvd = false;
-#if OPAL_ENABLE_FT_CR == 1
-    proc->ckpt_state = 0;
-    proc->ckpt_snapshot_ref = NULL;
-    proc->ckpt_snapshot_loc = NULL;
-#endif
+    proc->flags = 0;
+    OBJ_CONSTRUCT(&proc->attributes, opal_list_t);
 }
 
 static void orte_proc_destruct(orte_proc_t* proc)
 {
-    opal_pstats_t *stats;
-
-    /* do NOT free the nodename field as this is
-     * simply a pointer to a field in the
-     * associated node object - the node object
-     * will free it
-     */
-#if OPAL_HAVE_HWLOC
-    if (NULL != proc->cpu_bitmap) {
-        free(proc->cpu_bitmap);
-    }
-#endif
-
     if (NULL != proc->node) {
         OBJ_RELEASE(proc->node);
         proc->node = NULL;
     }
-    
+
     if (NULL != proc->rml_uri) {
         free(proc->rml_uri);
         proc->rml_uri = NULL;
     }
 
-    while (NULL != (stats = (opal_pstats_t*)opal_ring_buffer_pop(&proc->stats))) {
-        OBJ_RELEASE(stats);
-    }
-    OBJ_DESTRUCT(&proc->stats);
-
-#if OPAL_ENABLE_FT_CR == 1
-    if (NULL != proc->ckpt_snapshot_ref) {
-        free(proc->ckpt_snapshot_ref);
-        proc->ckpt_snapshot_ref = NULL;
-    }
-    if (NULL != proc->ckpt_snapshot_loc) {
-        free(proc->ckpt_snapshot_loc);
-        proc->ckpt_snapshot_loc = NULL;
-    }
-#endif
+    OPAL_LIST_DESTRUCT(&proc->attributes);
 }
 
 OBJ_CLASS_INSTANCE(orte_proc_t,
@@ -1016,9 +849,7 @@ static void orte_job_map_construct(orte_job_map_t* map)
     map->last_mapper = NULL;
     map->mapping = 0;
     map->ranking = 0;
-#if OPAL_HAVE_HWLOC
     map->binding = 0;
-#endif
     map->ppr = NULL;
     map->cpus_per_rank = 1;
     map->display_map = false;
@@ -1059,3 +890,43 @@ OBJ_CLASS_INSTANCE(orte_job_map_t,
                    opal_object_t,
                    orte_job_map_construct,
                    orte_job_map_destruct);
+
+static void orte_attr_cons(orte_attribute_t* p)
+{
+    p->key = 0;
+    p->local = true;  // default to local-only data
+    memset(&p->data, 0, sizeof(p->data));
+}
+static void orte_attr_des(orte_attribute_t *p)
+{
+    if (OPAL_BYTE_OBJECT == p->type) {
+        if (NULL != p->data.bo.bytes) {
+            free(p->data.bo.bytes);
+        }
+    } else if (OPAL_BUFFER == p->type) {
+        OBJ_DESTRUCT(&p->data.buf);
+    } else if (OPAL_STRING == p->type) {
+        free(p->data.string);
+    }
+}
+OBJ_CLASS_INSTANCE(orte_attribute_t,
+                   opal_list_item_t,
+                   orte_attr_cons, orte_attr_des);
+
+static void tcon(orte_topology_t *t)
+{
+    t->topo = NULL;
+    t->sig = NULL;
+}
+static void tdes(orte_topology_t *t)
+{
+    if (NULL != t->topo) {
+        hwloc_topology_destroy(t->topo);
+    }
+    if (NULL != t->sig) {
+        free(t->sig);
+    }
+}
+OBJ_CLASS_INSTANCE(orte_topology_t,
+                   opal_object_t,
+                   tcon, tdes);
