@@ -18,7 +18,7 @@
 #include "oshmem/mca/spml/base/base.h"
 #include "oshmem/mca/spml/yoda/spml_yoda_putreq.h"
 #include "oshmem/mca/spml/yoda/spml_yoda_getreq.h"
-#include "ompi/mca/btl/btl.h"
+#include "opal/mca/btl/btl.h"
 
 #define SPML_BASE_DO_CMP(res, addr, op, val) \
     switch((op)) { \
@@ -144,7 +144,7 @@ int mca_spml_base_wait(void* addr, int cmp, void* value, int datatype)
  */
 int mca_spml_base_wait_nb(void* handle)
 {
-    /* TODO fence is a gag for more accurate code 
+    /* TODO fence is a gag for more accurate code
      * Use shmem_quiet() (or a function calling shmem_quiet()) or
      * shmem_wait_nb() to force completion of transfers for non-blocking operations.
      */
@@ -158,3 +158,10 @@ int mca_spml_base_oob_get_mkeys(int pe, uint32_t seg, sshmem_mkey_t *mkeys)
     return OSHMEM_ERROR;
 }
 
+void mca_spml_base_rmkey_unpack(sshmem_mkey_t *mkey, int pe)
+{
+}
+
+void mca_spml_base_rmkey_free(sshmem_mkey_t *mkey)
+{
+}
